module.exports = function (db) {
    var util = require('../util');
    var dao = require('../dao')(db);
    var W = require('when');
    var fs = require('fs');
    return {
        create: function (req, res) {
            var attribs = {"name": req.body.name, "description": req.body.description, "t_description": req.body.t_description,
                "category": req.body.category, "coins": req.body.coins, "quantity": req.body.quantity};
            if (util.validateParams(res, {"company_id": req.session.company_id}) && util.validateParams(res, attribs)) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return W.all([dao.create(db.Product, attribs), company]);
                    })
                    .spread(function (product, company) {
                        if (product) return company.addProduct(product);
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        read: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else if (req.params.id) return company.getProducts({ where: {id: req.params.id} });
                        else return company.getProducts();
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        update: function (req, res) {
            var attribs = {"name": req.body.name, "description": req.body.description, "t_description": req.body.t_description,
                "category": req.body.category, "coins": req.body.coins, "quantity": req.body.quantity};

            if (util.validateParams(res, {"company_id": req.session.company_id}) && util.validateParams(res, attribs)) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return company.getProducts({ where: {id: req.params.id} });
                    })
                    .then(function (product) {
                        if (product.length == 0) util.reject("Product not exists = " + req.params.id);
                        else return product[0].updateAttributes(attribs);
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        delete: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return company.getProducts({ where: {id: req.params.id} });
                    })
                    .then(function (product) {
                        if (product.length == 0) util.reject("Product not exists = " + req.params.id);
                        else  return W.all([dao.getByWhere(db.ProductImage, {productid: product[0].id}), product[0]]);
                    })
                    .spread(function (productsImages, product) {
                        W.iterate(function (i) {
                            return i + 1;
                        }, function (i) {
                            // Stop when i >= length
                            return i >= productsImages.length;
                        }, function (i) {
                            fs.unlink('./public/uploads/productImage/' + productsImages[i].dataValues.id + productsImages[i].dataValues.ext);
                            dao.deleteById(db.ProductImage, productsImages[i].dataValues.id);
                        }, 0).done();
                        return product.destroy();
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },

        //SHOP
        readByCategory: function (req, res) {
            var where = {};
            if (req.params.id)where = {category: req.params.id};
            util.getUserIdByToken(req.params.idUser) //ID = access_token
                .then(function (userID) {
                    return W.all([dao.User.getByIdFacebook(userID), userID]);
                })
                .spread(function (user, userID) {
                    if (!user) util.reject("User not registered with id = " + userID);
                    else return dao.findAllWhereInclude(db.Product, where, [
                        { model: db.ProductImage, as: 'images'}
                    ])
                })
                .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                .done();
        },
        readShop: function (req, res) {
            util.getUserIdByToken(req.params.idUser) //ID = access_token
                .then(function (userID) {
                    return W.all([dao.User.getByIdFacebook(userID), userID]);
                })
                .spread(function (user, userID) {
                    if (!user) util.reject("User not registered with id = " + userID);
                    else return dao.findAllWhereInclude(db.Product, {id: req.params.id}, [
                        { model: db.ProductImage, as: 'images'}
                    ])
                })
                .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                .done();
        }
    }
};