module.exports = function (db) {
    var util = require('../util');
    var dao = require('../dao')(db);
    var W = require('when');
    var path = require('path');
    var fs = require('fs');
    return {
        create: function (req, res) {
            if(!req.files) util.stdErr400(res, "File undefined");
            else {
                var attribs = {"src": req.files.file.path, "ext": path.extname(req.files.file.name).toLowerCase()};
                if (util.validateParams(res, {"company_id": req.session.company_id}) && util.validateParams(res, attribs)) {
                    dao.getById(db.Company, req.session.company_id)
                        .then(function (company) {
                            if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                            else return company.getProducts({ where: {id: req.params.id} });
                        })
                        .then(function (products) {
                            if (products.length == 0) util.reject("Product not found with id = " + req.params.id);
                            else return W.all([dao.create(db.ProductImage,attribs), products]);
                        })
                        .spread(function (productImage, products) {
                            if (productImage) {
                                var tempPath = req.files.file.path;
                                var targetPath = path.resolve('./public/uploads/productImage/' + productImage.id + path.extname(req.files.file.name).toLowerCase());

                                if (path.extname(req.files.file.name).toLowerCase() === '.jpg'
                                    || path.extname(req.files.file.name).toLowerCase() === '.png')
                                    return W.all([fs.rename(tempPath, targetPath), products[0].addImage(productImage), 1]);
                                 else  return W.all([fs.unlink(tempPath), null, 2]);
                            }
                        })
                        .spread(function (fs, productImage, option) {
                            if (option == 1 && productImage) return productImage;
                            else if (option == 2)util.reject("Only .png and .jpg files are allowed!");
                        })
                        .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                        .done();
                }
            }
        },
        read: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return company.getProducts({ where: {id: req.params.id} });
                    })
                    .then(function (products) {
                        if (products.length == 0) util.reject("Product not found with id = " + req.params.id);
                        else if (req.params.idPhoto) return products[0].getImages({ where: {id: req.params.idPhoto} });
                        else return products[0].getImages();
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        },
        delete: function (req, res) {
            if (util.validateParams(res, {"company_id": req.session.company_id})) {
                dao.getById(db.Company, req.session.company_id)
                    .then(function (company) {
                        if (!company) util.reject("Company not registered with id = " + req.session.company_id);
                        else return company.getProducts({ where: {id: req.params.id} });
                    })
                    .then(function (products) {
                        if (products.length == 0) util.reject("Product not found with id = " + req.params.id);
                        else return products[0].getImages({ where: {id: req.params.idPhoto} });
                    })
                    .then(function (photo) {
                        if (photo.length == 0) util.reject("Photo not found with id = " + req.params.idPhoto);
                        else {
                            fs.unlink('./public/uploads/productImage/' + photo[0].dataValues.id + photo[0].dataValues.ext);
                            photo[0].destroy();
                        }
                    })
                    .then(util.stdSeqSuccess.genFuncLeft(res), util.stdSeqError.genFuncLeft(res))
                    .done();
            }
        }
    }
};